#include <OS.hpp>
#include <string>
#include "Clock.h"

avi::connection::Clock::Clock() = default;

void
avi::connection::Clock::setTimeFrame(avi::connection::TimeFrame frame)
{
    fullFrame = frame;
    setAvailableTimeFrame(TimeFrame(0, 0));
    setTargetTime(0);
}

void avi::connection::Clock::setRevisionFrame(long firstRevision, long lastRevision) {
    this->firstRevision = firstRevision;
    this->lastRevision = lastRevision;
}

avi::connection::TimeFrame
avi::connection::Clock::getTimeFrame()
{
    return fullFrame;
}

void
avi::connection::Clock::setAvailableTimeFrame(avi::connection::TimeFrame frame)
{
    availableFrame = frame;
}

avi::connection::TimeFrame
avi::connection::Clock::getAvailableTimeFrame()
{
    return availableFrame;
}

void
avi::connection::Clock::advance(long time, bool absolute)
{
    float currentSpeed = speed;
    if(absolute) currentSpeed = 1;

    if ((fullFrame.begin + targetTime + (long)((long double)currentSpeed * time) <= fullFrame.end) &&
        (fullFrame.begin + targetTime + (long)((long double)currentSpeed * time) >= fullFrame.begin))
    {
        targetTime += (long)((long double)currentSpeed * time);
    }

    if(loopBack) checkLoopBack();
}

void avi::connection::Clock::setTargetRevision(long revision) {
    if(revision >= lastRevision || revision < firstRevision) {
        godot::PoolStringArray poolStringArray;
        poolStringArray.push_back("Invalid Revision!");
        std::string notificationString = "Revision " + std::to_string(revision) + " is not in range [" + std::to_string(firstRevision) + "," + std::to_string(lastRevision) + ")";
        poolStringArray.push_back(notificationString.c_str());
        godot::OS::get_singleton()->execute("notify-send", poolStringArray, false);
        return;
    }

    targetRevision = revision;
    targetRevisionUpdate = true;
}

void avi::connection::Clock::updateTargetRevision(long revision) {
    targetRevision = revision;
}



void
avi::connection::Clock::setTargetTime(long time)
{
    if (time <= fullFrame.end)
    {
        targetTime = time;
    }
    else
    {
        if (targetTime + 1 <= fullFrame.end)
        {
            targetTime++;
        }
    }
}

long
avi::connection::Clock::getTargetTime() const
{
    return targetTime;
}

long avi::connection::Clock::getTargetRevision() const {
    return targetRevision;
}

void
avi::connection::Clock::setSpeed(float add)
{
    speed = add;
}

float
avi::connection::Clock::getSpeed() const
{
    return speed;
}

void avi::connection::Clock::setLoopBack(bool loopBack) {
    this->loopBack = loopBack;
}

void avi::connection::Clock::checkLoopBack() {
    if(speed > 0 && (fullFrame.end - (targetTime + fullFrame.begin)) < 100000) {
        setTargetTime(0);
    }

    if(speed < 0 && targetTime < 100000) {
        setTargetTime(fullFrame.end - fullFrame.begin);
    }
}

bool avi::connection::Clock::isBuffering() {
    return buffering;
}

void avi::connection::Clock::setBuffering(bool status) {
    buffering = status;
}

bool avi::connection::Clock::hasRevisionUpdate() {
    bool update = targetRevisionUpdate;
    targetRevisionUpdate = false;
    return update;
}