#pragma once

#include <optional>

#include <Quat.hpp>

#include <Vector3.hpp>
#include <arviz_godot/lib/avi/connection/InteractionType.h>
#include <arviz_godot/lib/avi/core/Element.h>

namespace avi::connection
{
    class Interaction
    {
    public:
        enum Transformation
        {
            TRANSFORMATION_BEGIN,
            TRANSFORMATION_DURING,
            TRANSFORMATION_END
        };

        static avi::connection::Interaction createSelection(avi::core::Element* element);

        static avi::connection::Interaction createDeselection(avi::core::Element* element);

        static avi::connection::Interaction createOption(avi::core::Element* element,
                                                         int64_t option);

        static avi::connection::Interaction createManipulation(avi::core::Element* element,
                                                               godot::Vector3 translation,
                                                               godot::Quat rotation,
                                                               godot::Vector3 scale,
                                                               Transformation transformation);

        InteractionType getType();

        std::optional<int64_t> getOption();

        godot::Vector3 getTranslation();

        godot::Quat getRotation();

        godot::Vector3 getScale();

        avi::core::Element* getElement();

        Transformation getTransformation();

    private:
        InteractionType interactionType;
        Transformation transformation;

        godot::Vector3 translation;
        godot::Quat rotation;
        godot::Vector3 scale;
        avi::core::Element* element;

        std::optional<int64_t> option;
    };

} // namespace avi::connection
