template <class K, class V>
void
avi::core::Cache<K, V>::emplace(K key, V value)
{
    if (!contains(key))
    {
        if (entries.size() >= CACHE_SIZE)
        {
            std::optional<K> victimKey;
            uint64_t victimLastUse = std::numeric_limits<uint64_t>::max();

            for (auto const& [currentKey, entry] : entries)
            {
                auto const& [o, lastUse] = entry;

                if (lastUse < victimLastUse)
                {
                    victimKey = currentKey;
                    victimLastUse = lastUse;
                }
            }

            if (victimKey.has_value())
            {
                std::optional<V> victim = get(victimKey.value());
                entries.erase(victimKey.value());
            }
        }

        entries.emplace(key, Entry{value, counter});
        counter++;
    }
}

template <class K, class V>
bool
avi::core::Cache<K, V>::contains(K key)
{
    return entries.find(key) != entries.end();
}

template <class K, class V>
std::optional<V>
avi::core::Cache<K, V>::get(K key)
{
    auto found = entries.find(key);

    if (found != entries.end())
    {
        auto& [k, entry] = *found;
        auto& [value, lastUse] = entry;

        lastUse = counter;

        return {value};
    }

    return std::nullopt;
}

template <class K, class V>
void
avi::core::Cache<K, V>::clear()
{
    entries.clear();
}
