#pragma once

#include <VirtualRobot/Import/MeshImport/AssimpReader.h>

#include <Mesh.hpp>
#include <Shape.hpp>
#include <SpatialMaterial.hpp>
#include <arviz_godot/lib/avi/core/Cache.h>

namespace avi::core
{
    class ModelLoader
    {
    public:
        VirtualRobot::ManipulationObjectPtr getObject(const std::string& name);

        godot::Ref<godot::Mesh> loadModel(const std::string& name, std::string& fullName);
        godot::Ref<godot::Mesh> loadModelFromFile(const std::string& name);

        avi::core::Cache<godot::Mesh*, godot::Ref<godot::Shape>>& getShapeCache();
        avi::core::Cache<VirtualRobot::ManipulationObjectPtr, godot::Ref<godot::Shape>>&
        getCollisionShapeCache();

        ModelLoader();

        static godot::Ref<godot::Mesh>
        createObjectFromTriMesh(std::shared_ptr<VirtualRobot::TriMeshModel> model);

        void clearCaches();

    private:
        godot::Ref<godot::Mesh> loadModelFromNonObjFile(const std::string& name);

        godot::Ref<godot::Mesh> error();

        avi::core::Cache<std::string, godot::Ref<godot::Mesh>> modelCache;
        avi::core::Cache<godot::Mesh*, godot::Ref<godot::Shape>> shapeCache;
        avi::core::Cache<VirtualRobot::ManipulationObjectPtr, godot::Ref<godot::Shape>>
            collisionShapeCache;
        avi::core::Cache<std::string, VirtualRobot::ManipulationObjectPtr> objectCache;

        godot::Mesh* errorMesh;
    };
} // namespace avi::core
