#pragma once

#include <Godot.hpp>
#include <arviz_godot/lib/avi/core/OptionFile.h>
#include <arviz_godot/lib/avi/core/ValueOption.h>

namespace avi::core
{
    template <typename O>
    class OptionFileReader
    {
    public:
        explicit OptionFileReader(O& optionContainer);

        void read(avi::core::OptionFile& optionFile);

        template <typename V>
        void transform(avi::core::ValueOption<V, OptionFileReader>& option);

    private:
        template <typename V>
        static void
        transform(avi::core::ValueOption<V, OptionFileReader>& option,
                  avi::core::OptionFile& optionFile)
        {
            auto v = optionFile.template getValue<V>(option.getKey());
            if (v.has_value())
            {
                option.setValue(v.value());
            }
        }

    private:
        O& optionContainer;
        avi::core::OptionFile* currentOptionFile{};
    };

    template <typename O>
    OptionFileReader<O>::OptionFileReader(O& optionContainer) : optionContainer(optionContainer)
    {
    }

    template <typename O>
    void
    OptionFileReader<O>::read(OptionFile& optionFile)
    {
        currentOptionFile = &optionFile;

        for (auto& option : optionContainer.template getOptions<OptionFileReader<O>>())
        {
            option->accept(*this);
        }

        currentOptionFile = nullptr;
    }

    template <typename O>
    template <typename V>
    void
    OptionFileReader<O>::transform(ValueOption<V, OptionFileReader>& option)
    {
        transform(option, *currentOptionFile);
    }
} // namespace avi::core
