#pragma once

#include <string>

#include <Godot.hpp>

#define STOPWATCH_START(name) auto start_##name = std::chrono::high_resolution_clock::now()

#define STOPWATCH_END(name)                                                                        \
    auto finish_##name = std::chrono::high_resolution_clock::now();                                \
    do                                                                                             \
    {                                                                                              \
        std::chrono::duration<double> elapsed = finish_##name - start_##name;                      \
        godot::Godot::print(                                                                       \
            (std::string("Stopwatch[") + #name + "] -> " + std::to_string(elapsed.count()))        \
                .c_str());                                                                         \
    } while (0)

namespace avi::core::utilities
{
    real_t
    signed_angle(const godot::Vector3& from, const godot::Vector3& to, const godot::Vector3& axis);

    godot::Basis scaled_local(const godot::Basis& basis, const godot::Vector3& scale);

    bool is_equal_approx(const godot::Vector3 a, const godot::Vector3 b);

    void print(const std::string& text);

} // namespace avi::core::utilities
