#include "CameraModeMenu.h"

void
avi::gui::CameraModeMenu::_register_methods()
{
    godot::register_method("_ready", &avi::gui::CameraModeMenu::_ready);
    godot::register_method("_process", &avi::gui::CameraModeMenu::_process);
}

void
avi::gui::CameraModeMenu::_init()
{
}

void
avi::gui::CameraModeMenu::_process()
{
    setActiveManipulator();
}

void
avi::gui::CameraModeMenu::_ready()
{
    this->modeCount = this->get_item_count();
}

void
avi::gui::CameraModeMenu::setup(avi::core::EventInvokingSystem& invokingSystem,
                                avi::core::EventSubscriptionSystem& eventSubscriptionSystem,
                                avi::core::EventToKeyProvider& eventToKeyProvider)
{
    this->events = &invokingSystem;

    auto setupButton =
        [this, &eventSubscriptionSystem, &eventToKeyProvider](const char* event, int index)
    {
        eventSubscriptionSystem.subscribe(event, [this, index]() -> void { this->select(index); });

        std::string formattedText = this->get_item_text(index).ascii().get_data();
        if (not formattedText.empty())
        {
            formattedText += " ";
        }

        std::optional<godot::InputEventKey*> key = eventToKeyProvider.getEventKey(event);

        if (key.has_value())
        {
            formattedText += "[";
            formattedText += key.value()->as_text().ascii().get_data();
            formattedText += "]";
        }

        this->set_item_text(index, formattedText.c_str());
    };

    setupButton("CAMERA_TECHNICAL", CAMERA_TECHNICAL);
    setupButton("CAMERA_PLAYFUL", CAMERA_PLAYFUL);
}

int
avi::gui::CameraModeMenu::getSelectedItemID()
{
    for (int i = 0; i < modeCount; i++)
    {
        if (this->is_selected(i))
        {
            return i;
        }
    }

    return -1;
}

void
avi::gui::CameraModeMenu::setActiveManipulator()
{
    int activeManipulatorID = getSelectedItemID();

    switch (activeManipulatorID)
    {
        case CAMERA_TECHNICAL:
            avi::gui::CameraModeMenu::disableActiveItemText();

            if (current != CAMERA_TECHNICAL)
                events->invoke("VIEW_SCHEME_SWITCH");
            current = CAMERA_TECHNICAL;

            break;

        case CAMERA_PLAYFUL:
            avi::gui::CameraModeMenu::disableActiveItemText();

            if (current != CAMERA_PLAYFUL)
                events->invoke("VIEW_SCHEME_SWITCH");
            current = CAMERA_PLAYFUL;

            break;

        default:
            break;
    }
}

void
avi::gui::CameraModeMenu::disableActiveItemText()
{
}
