#pragma once

#include <mutex>

#include <Button.hpp>
#include <Control.hpp>
#include <Godot.hpp>
#include <LineEdit.hpp>
#include <PackedScene.hpp>
#include <ResourceLoader.hpp>
#include <Texture.hpp>
#include <Tree.hpp>
#include <arviz_godot/lib/avi/scene/VisualizedScene.h>

namespace avi::gui
{
    class LayerSelection : public godot::Control
    {
        GODOT_CLASS(LayerSelection, Control)

    public:
        virtual ~LayerSelection() = default;

        static void _register_methods();

        void _init();

        void _ready();

        void setup(avi::scene::VisualizedScene& scene);

        void clear();

        void lock();

        void unlock();

    private:
        void update_layers();

        void updateActiveComponents();

        void setVisibility(bool visible, int64_t component, int64_t layer);

        void setVisibilityAll(bool visible);

        void setCollapseAll(bool collapsed);

        void filterLayers();

        void buildTree();

        void setFilterVisibility(bool visible);

        struct LayerTreeItem
        {
            godot::TreeItem* treeItem;
            int componentIndex;
            int layerIndex;
            bool active;
        };

        struct ComponentTreeItem
        {
            godot::TreeItem* treeItem;
            bool previousState;
        };

    private:
        godot::Ref<godot::PackedScene> itemScene;
        godot::Ref<godot::PackedScene> layerScene;

        godot::Tree* tree{};
        godot::TreeItem* treeRootNode{};
        std::vector<ComponentTreeItem> componentTreeItems{};
        std::vector<LayerTreeItem> layerTreeItems{};

        godot::Control* container{nullptr};
        godot::Control* hideAllButton{nullptr};
        godot::Control* showAllButton{nullptr};

        godot::Control* collapseAllButton{nullptr};
        godot::Control* expandAllButton{nullptr};

        godot::LineEdit* filterTextBox{nullptr};
        godot::Control* hideFilteredButton{nullptr};
        godot::Control* showFilteredButton{nullptr};

        std::vector<std::string> components{};
        std::vector<std::vector<std::string>> layers{};
        avi::scene::VisualizedScene* scene{nullptr};

        std::mutex mutex;
    };
} // namespace avi::gui
