#include "ManipulationMenu.h"

#include <InputEvent.hpp>

void
avi::gui::ManipulationMenu::_register_methods()
{
    godot::register_method("_ready", &avi::gui::ManipulationMenu::_ready);
    godot::register_method("_process", &avi::gui::ManipulationMenu::_process);
}

void
avi::gui::ManipulationMenu::_init()
{
}

void
avi::gui::ManipulationMenu::setup(avi::interaction::Manipulator& manipulator,
                                  avi::core::EventSubscriptionSystem& eventSubscriptionSystem,
                                  avi::core::EventToKeyProvider& eventToKeyProvider)
{
    this->manipulator = &manipulator;

    auto setupButton =
        [this, &eventSubscriptionSystem, &eventToKeyProvider](const char* event, int index)
    {
        eventSubscriptionSystem.subscribe(event, [this, index]() -> void { this->select(index); });

        std::string formattedText = this->get_item_text(index).ascii().get_data();
        if (not formattedText.empty())
        {
            formattedText += " ";
        }

        std::optional<godot::InputEventKey*> key = eventToKeyProvider.getEventKey(event);

        if (key.has_value())
        {
            formattedText += "[";
            formattedText += key.value()->as_text().ascii().get_data();
            formattedText += "]";
        }

        this->set_item_text(index, formattedText.c_str());
    };

    setupButton("MANIPULATOR_NONE", MANIPULATOR_NONE);
    setupButton("MANIPULATOR_MOVE", MANIPULATOR_MOVE);
    setupButton("MANIPULATOR_ROTATE", MANIPULATOR_ROTATE);
    setupButton("MANIPULATOR_SCALE", MANIPULATOR_SCALE);
}

void
avi::gui::ManipulationMenu::_process()
{
    setActiveManipulator();
}

void
avi::gui::ManipulationMenu::_ready()
{
    this->manipulatorCount = this->get_item_count();
}

int
avi::gui::ManipulationMenu::getSelectedItemID()
{
    for (int i = 0; i < manipulatorCount; i++)
    {
        if (this->is_selected(i))
        {
            return i;
        }
    }

    return -1;
}

void
avi::gui::ManipulationMenu::setActiveManipulator()
{
    int activeManipulatorID = getSelectedItemID();

    switch (activeManipulatorID)
    {
        case MANIPULATOR_NONE:
            selectionMode = ManipulationMode::NONE;
            manipulator->setManipulationMode(selectionMode);
            break;
        case MANIPULATOR_MOVE:
            selectionMode = ManipulationMode::TRANSLATE;
            manipulator->setManipulationMode(selectionMode);
            break;
        case MANIPULATOR_ROTATE:
            selectionMode = ManipulationMode::ROTATE;
            manipulator->setManipulationMode(selectionMode);
            break;
        case MANIPULATOR_SCALE:
            selectionMode = ManipulationMode::SCALE;
            manipulator->setManipulationMode(selectionMode);
            break;
        default:
            break;
    }
}
