#pragma once

#include <Control.hpp>
#include <Godot.hpp>
#include <ItemList.hpp>
#include <Label.hpp>
#include <Texture.hpp>
#include <arviz_godot/lib/avi/connection/Connectable.h>
#include <arviz_godot/lib/avi/connection/StorageModeSwitch.h>
#include <arviz_godot/lib/avi/core/ControlAdvertiser.h>
#include <arviz_godot/lib/avi/recording/VideoRecorder.h>

namespace avi::gui
{
    class StatusIcons : public godot::Control, public core::ControlAdvertiser
    {
        GODOT_CLASS(StatusIcons, Control)

    public:
        virtual ~StatusIcons() = default;

        static void _register_methods();

        void _init();

        void _process(float delta);

        using avi::core::ControlAdvertiser::setup;

        void setup(avi::connection::Connectable& source,
                   avi::connection::StorageModeSwitch& mode,
                   avi::recording::VideoRecorder& recorder,
                   avi::scene::GraphicsController& graphicsController,
                   avi::core::EventToKeyProvider& eventToKeyProvider);

    protected:
        void updateAdvertisements(
            const std::vector<std::reference_wrapper<const std::string>>& ads) override;

    private:
        void updateTextColor();

        avi::connection::Connectable* source{};
        avi::connection::StorageModeSwitch* mode{};
        avi::recording::VideoRecorder* recorder{};
        avi::scene::GraphicsController* graphicsController{};

        enum connectionMode
        {
            DISCONNECTED,
            CONNECTED,
            REPLAY,
        };

        godot::Ref<godot::Texture> disconnectedTexture;
        godot::Ref<godot::Texture> connectedTexture;
        godot::Ref<godot::Texture> replayTexture;
        godot::Ref<godot::Texture> recordingTexture;
        godot::Ref<godot::Texture> replayRecordingTexture;

        connectionMode currentMode{};
        bool isVideoRecording{};
        bool isReplayRecording{};

        godot::ItemList* itemList{};
        std::string advertisement{};
    };
} // namespace avi::gui
