#pragma once

#include <Control.hpp>
#include <Godot.hpp>
#include <Popup.hpp>
#include <arviz_godot/lib/avi/connection/Connectable.h>
#include <arviz_godot/lib/avi/core/EventInvokingSystem.h>
#include <arviz_godot/lib/avi/core/EventToKeyProvider.h>
#include <arviz_godot/lib/avi/core/InterfaceUtility.h>
#include <arviz_godot/lib/avi/core/Settings.h>
#include <arviz_godot/lib/avi/gui/GraphicsSettings.h>
#include <arviz_godot/lib/avi/gui/LayerSelection.h>
#include <arviz_godot/lib/avi/gui/ManipulationMenu.h>
#include <arviz_godot/lib/avi/gui/OrientationWindow.h>
#include <arviz_godot/lib/avi/gui/ReplayCockpit.h>
#include <arviz_godot/lib/avi/gui/SelectionMenu.h>
#include <arviz_godot/lib/avi/gui/SettingsWindow.h>
#include <arviz_godot/lib/avi/gui/StatusIcons.h>
#include <arviz_godot/lib/avi/interaction/InteractionDispatch.h>
#include <arviz_godot/lib/avi/recording/ImageRecorder.h>
#include <arviz_godot/lib/avi/replay/ReplayController.h>
#include <arviz_godot/lib/avi/scene/GraphicsController.h>
#include <arviz_godot/lib/avi/scene/ProfileStorage.h>

namespace avi::gui
{
    class UserInterface : public godot::Control, public avi::core::InterfaceUtility
    {
        GODOT_CLASS(UserInterface, Control)

    public:
        virtual ~UserInterface() = default;

        static void _register_methods();

        void _init();

        void _ready();

        void setupUI(avi::core::EventInvokingSystem& eventInvokingSystem,
                     avi::core::EventSubscriptionSystem& eventSubscriptionSystem,
                     avi::core::EventToKeyProvider& eventToKeyProvider,
                     avi::core::Settings& settings);

        void setupStatusIcons(avi::connection::Connectable& connectable,
                              avi::connection::StorageModeSwitch& mode,
                              avi::recording::VideoRecorder& recorder,
                              avi::scene::GraphicsController& graphicsController,
                              avi::core::EventToKeyProvider& eventToKeyProvider);

        void setupScene(avi::scene::VisualizedScene& visualizedScene,
                        avi::recording::VideoRecorder& videoRecorder,
                        avi::recording::ImageRecorder& imageRecorder);

        void setupGraphics(avi::scene::GraphicsController& graphicsController,
                           avi::scene::ProfileStorage& profileStorage);

        void setupInteraction(avi::interaction::InteractionDispatch& interactionDispatch);

        void setupManipulation(avi::core::EventToKeyProvider& eventToKeyProvider);

        void setupReplay(avi::replay::ReplayController& replayController,
                         avi::scene::VisualizedScene& visualizedScene);

        void requestHiddenUI() override;
        void requestVisibleUI() override;

        void openSelectionMenu(godot::Vector2 mousePosition,
                               std::vector<std::string> interactions,
                               std::function<void(int64_t)> callback);

        void openTextInputPrompt();

        void clear();

        void lockLayerSelection();
        void unlockLayerSelection();

        avi::core::ControlAdvertiser& getAdvertiser();

    private:
        void setupConstantHelpBar(avi::core::EventToKeyProvider& eventToKeyProvider);

        void _scene_menu_select(int64_t id);
        void _view_menu_select(int64_t id);

        bool hiddenToggle = false;
        uint64_t hidingCounter = 0;

        std::vector<std::string> viewMenuOptions;

        avi::gui::LayerSelection* layerSelection;
        avi::gui::GraphicsSettings* graphicsSettings;
        avi::gui::ReplayCockpit* replayCockpit;

        avi::core::Settings* settings{};
        avi::core::EventInvokingSystem* eventInvokingSystem{};
        avi::core::EventSubscriptionSystem* eventSubscriptionSystem{};
        avi::gui::SettingsWindow* settingsWindow;
        avi::gui::OrientationWindow* orientationWindow;

        avi::recording::VideoRecorder* videoRecorder{};
        avi::recording::ImageRecorder* imageRecorder{};

        avi::connection::Connectable* connectable{};

        avi::interaction::InteractionDispatch* interactionDispatch{};

        avi::gui::SelectionMenu* selectionMenu;

        avi::gui::ManipulationMenu* manipulationMenu;
        godot::Popup* helpWindow;

        avi::core::ControlAdvertiser* advertiser;
    };
} // namespace avi::gui
