#pragma once

#include "Move.h"
#include "None.h"
#include "Rotate.h"
#include "Scale.h"
#include <Node.hpp>
#include <arviz_godot/lib/avi/connection/Interaction.h>
#include <arviz_godot/lib/avi/core/ControlAdvertiser.h>
#include <arviz_godot/lib/avi/interaction/ManipulationMode.h>
#include <arviz_godot/lib/avi/scene/ElementNode.h>

namespace avi::interaction
{
    class InteractionDispatch;

    class Manipulator
    {
    public:
        Manipulator(avi::interaction::InteractionDispatch& dispatch);

        void setupControlAdvertisement(avi::core::ControlAdvertiser& advertiser);

        void setManipulationMode(ManipulationMode& mode);

        ManipulationMode getManipulationMode();

        void target(avi::scene::ElementNode& element);

        void untarget();

        void setGizmos(avi::interaction::Move& move,
                       avi::interaction::Rotate& rotate,
                       avi::interaction::Scale& scale,
                       avi::interaction::None& none);

        void sendManipulation();
        void endManipulation();

        void setGizmoTransform(godot::Spatial* gizmo, godot::Transform global, bool isLocal);
        void scaleGizmo(godot::Spatial* gizmo);

        void preDelete();

    private:
        avi::connection::Interaction
        createInteraction(avi::connection::Interaction::Transformation transformation);

    private:
        avi::interaction::InteractionDispatch* dispatch{};
        avi::scene::ElementNode* targetNode{};
        godot::Spatial* proxy;
        ManipulationMode manipulationMode;

        avi::core::ControlAdvertiser* advertiser;
        size_t adID;

        avi::interaction::Move* move{};
        avi::interaction::Rotate* rotate{};
        avi::interaction::Scale* scale{};
        avi::interaction::None* none{};

        godot::Node* gizmoNodeParent{};

        bool didTransform = false;

        void deactivateAllGizmos();
    };
} // namespace avi::interaction
