#include "ImageRecorder.h"

#include <ArmarXCore/core/logging/Logging.h>

#include <ArmarXCore/core/time/DateTime.h>

avi::recording::ImageRecorder::ImageRecorder(scene::CameraSet& cameras,
                                             core::SettingsProvider& settings,
                                             core::EventSubscriptionSystem& events) :
    cameras(cameras), settings(settings)
{
    events.subscribe("TAKE_SCREENSHOT", [this]() { this->takeScreenshot(); });
}

void
avi::recording::ImageRecorder::takeScreenshot()
{
    auto* camera = cameras.getActiveCamera();
    auto screenContent = camera->getCurrentScreenContent();

    std::filesystem::path directory = settings.getScreenshotDirectory();
    std::filesystem::create_directories(directory);

    armarx::DateTime now = armarx::DateTime::Now();
    std::string time = now.toDateTimeString();
    std::string filename = "avi_" + time + ".png";

    std::filesystem::path path = directory / filename;
    lastScreenshotPath = path;
    screenContent.image->save_png(path.string().c_str());

    godot::OS* os = godot::OS::get_singleton();
    godot::PoolStringArray poolStringArray;
    poolStringArray.push_back("Screenshot saved! Click 'Scene' > 'Open Screenshot Directory'");
    poolStringArray.push_back(settings.getScreenshotDirectory().c_str());

    os->execute("notify-send", poolStringArray, false);
}

std::filesystem::path
avi::recording::ImageRecorder::getLastScreenshotPath()
{
    std::filesystem::path path = settings.getScreenshotDirectory();
    if (!lastScreenshotPath.empty())
    {
        path = lastScreenshotPath;
    }

    return path;
}
