#include "ReplayController.h"

avi::replay::ReplayController::ReplayController(
    avi::connection::StorageModeSwitch& storageModeSwitch,
    avi::connection::Connectable& connectable) :
    storageModeSwitch(storageModeSwitch), connectable(connectable)
{
}

std::string
avi::replay::ReplayController::startRecording(const std::string& name)
{
    std::string returnString = storageModeSwitch.recordReplay(name);

    if(returnString != "NOT_CONNECTED") recording = true;

    return returnString;
}

void
avi::replay::ReplayController::endRecording()
{
    storageModeSwitch.endRecording();
    recording = false;
}

std::vector<std::string>
avi::replay::ReplayController::getNames()
{
    return storageModeSwitch.getAllReplays();
}

void
avi::replay::ReplayController::startReplay(const std::string& replay)
{
    clock = std::make_shared<avi::connection::Clock>();
    storageModeSwitch.useReplayTiming(replay, clock);
    replayMode = true;
}

void
avi::replay::ReplayController::endReplay()
{
    storageModeSwitch.useLiveTiming();
    replayMode = false;
    playing = false;
}

void
avi::replay::ReplayController::setTime(long time)
{
    clock->setTargetTime(time);
}

void avi::replay::ReplayController::setRevision(long revision) {
    clock->setTargetRevision(revision);
}

void
avi::replay::ReplayController::setSpeed(float speed)
{
    this->speed = speed;
    if (playing)
    {
        clock->setSpeed(this->speed);
    }
}

long
avi::replay::ReplayController::getTargetTime()
{
    return clock->getTargetTime();
}

long avi::replay::ReplayController::getTargetRevision() {
    return clock->getTargetRevision();
}

void
avi::replay::ReplayController::addSpeed(float add)
{
    this->speed += add;
    if (playing)
    {
        clock->setSpeed(this->speed);
    }
}

long
avi::replay::ReplayController::getMaxTime()
{
    return clock->getTimeFrame().end;
}

long
avi::replay::ReplayController::getMinTime()
{
    return clock->getTimeFrame().begin;
}

float
avi::replay::ReplayController::getSpeed() const
{
    return this->speed;
}

void
avi::replay::ReplayController::pausePlayback()
{
    playing = false;
    clock->setSpeed(0.0);
}

void
avi::replay::ReplayController::resumePlayback()
{
    playing = true;
    if (speed == 0)
    {
        speed = 1.0;
    }
    clock->setSpeed(speed);
}

bool
avi::replay::ReplayController::getRecordingStatus() const
{
    return recording;
}

bool
avi::replay::ReplayController::inReplayMode() const
{
    return replayMode;
}

bool
avi::replay::ReplayController::isPlaying() const
{
    return playing;
}

bool
avi::replay::ReplayController::isConnected()
{
    return connectable.isConnected();
}

avi::connection::StorageModeSwitch::recordingInfo
avi::replay::ReplayController::getRecordingInformation(const std::string name)
{
    return storageModeSwitch.getRecordingInformation(name);
}
std::filesystem::path
avi::replay::ReplayController::getReplayPath()
{
    return storageModeSwitch.getRecordingPath();
}

void avi::replay::ReplayController::setLoopBack(bool loopBack) {
    clock->setLoopBack(loopBack);
}

bool avi::replay::ReplayController::isBuffering() {
    return clock->isBuffering();
}
