#pragma once

#include <filesystem>

#include <RobotAPI/interface/ArViz/Component.h>

#include <OptionButton.hpp>
#include <arviz_godot/lib/avi/connection/Connectable.h>
#include <arviz_godot/lib/avi/connection/StorageModeSwitch.h>

namespace avi::replay
{
    class ReplayController
    {
    public:
        explicit ReplayController(avi::connection::StorageModeSwitch& storageModeSwitch,
                                  avi::connection::Connectable& connectable);

        std::string startRecording(const std::string& name);

        void endRecording();

        std::vector<std::string> getNames();

        void startReplay(const std::string& replay);

        void endReplay();

        void setTime(long time);

        void setRevision(long revision);

        void setSpeed(float speed);

        void addSpeed(float add);

        bool isConnected();

        [[nodiscard]] float getSpeed() const;

        long getTargetTime();

        long getTargetRevision();

        long getMaxTime();

        long getMinTime();

        std::filesystem::path getReplayPath();

        void pausePlayback();

        void resumePlayback();

        void setLoopBack(bool loopBack);

        connection::StorageModeSwitch::recordingInfo
        getRecordingInformation(const std::string name);

        [[nodiscard]] bool getRecordingStatus() const;

        [[nodiscard]] bool inReplayMode() const;

        [[nodiscard]] bool isPlaying() const;

        bool isBuffering();

    private:
        avi::connection::StorageModeSwitch& storageModeSwitch;
        avi::connection::Connectable& connectable;

        std::shared_ptr<avi::connection::Clock> clock;

        float speed = 1.0;

        bool playing = false;

        bool recording = false;

        bool replayMode = false;
    };
} // namespace avi::replay
