#ifndef ARVIZ_INTERACTIVE_PROJECT_SOURCE_ARVIZ_INTERACTIVE_PROJECT_COMPONENTS_ARVIZ_INTERACTIVE_LIB_AVI_SCENE_ARROWELEMENTNODE_H_
#define ARVIZ_INTERACTIVE_PROJECT_SOURCE_ARVIZ_INTERACTIVE_PROJECT_COMPONENTS_ARVIZ_INTERACTIVE_LIB_AVI_SCENE_ARROWELEMENTNODE_H_

#include "ElementNode.h"
#include <CollisionShape.hpp>
#include <CylinderMesh.hpp>
#include <CylinderShape.hpp>
#include <Mesh.hpp>
#include <MeshInstance.hpp>
#include <Spatial.hpp>
#include <StaticBody.hpp>

namespace avi::scene
{
    class ArrowElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(ArrowElementNode, ElementNode)

    public:
        ArrowElementNode();
        ~ArrowElementNode() override = default;

        void _init();

        static ArrowElementNode* newArrowElementNode(avi::connection::SentElementType type,
                                                     std::string name,
                                                     avi::core::Layer* layer,
                                                     int flag,
                                                     std::vector<std::string> interactions,
                                                     godot::Vector3 position,
                                                     godot::Vector3 scale,
                                                     godot::Quat rotation,
                                                     godot::Color color,
                                                     float length,
                                                     float width);

        static ArrowElementNode* newArrowElementNode();

        static void _register_methods();

        void _ready();
        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        void setPosition(godot::Vector3 position);

        void setScale(godot::Vector3 scale);

        void setRotation(godot::Quat rotation);

        void setColor(godot::Color color);

        void setLength(float length);

        void setWidth(float width);

        void setShape();

        godot::Vector3 getPosition();

        godot::Vector3 getRotation();

        godot::Vector3 getScale();

        void setAllAttributes(avi::connection::SentElementType type,
                              std::string name,
                              avi::core::Layer* layer);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color,
                              float length,
                              float width);

    private:
        godot::MeshInstance* cylinder;
        godot::MeshInstance* cone;
        godot::StaticBody* staticBody;
        godot::CylinderMesh* cylinderMesh;
        godot::CollisionShape* collisionShape;

        bool shapeChecked = false;

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };
} // namespace avi::scene

#endif //ARVIZ_INTERACTIVE_PROJECT_SOURCE_ARVIZ_INTERACTIVE_PROJECT_COMPONENTS_ARVIZ_INTERACTIVE_LIB_AVI_SCENE_ARROWELEMENTNODE_H_
