#include "CylinderElementNode.h"

#include <utility>

#include "Converter.h"
#include <CylinderShape.hpp>

void
avi::scene::CylinderElementNode::_register_methods()
{
    register_method("_ready", &CylinderElementNode::_ready);
}

void
avi::scene::CylinderElementNode::_ready()
{
}

void
avi::scene::CylinderElementNode::_init()
{
}

void
avi::scene::CylinderElementNode::update(IceInternal::Handle<armarx::viz::data::Element> elementInfo)
{
    auto cylinder =
        IceInternal::Handle<armarx::viz::data::ElementCylinder>::dynamicCast(elementInfo);

    if (!godot::Math::is_equal_approx(height, cylinder->height) ||
        !godot::Math::is_equal_approx(radius, cylinder->radius))
    {
        radius = cylinder->radius;
        height = cylinder->height;

        getMeshInstance()->set_scale({radius, height / 2, radius});
        getCollisionShape()->set_scale({radius, height / 2, radius});
    }

    updateAttributes(cylinder->interaction.enableFlags,
                     cylinder->interaction.contextMenuOptions,
                     avi::scene::Converter::getPosition(cylinder->pose),
                     avi::scene::Converter::getScale(cylinder->scale),
                     avi::scene::Converter::getRotation(cylinder->pose),
                     avi::scene::Converter::getColor(cylinder->color));
}

avi::scene::CylinderElementNode*
avi::scene::CylinderElementNode::newCylinderElementNode(std::string name, avi::core::Layer* layer)
{
    CylinderElementNode* node = CylinderElementNode::_new();
    godot::Mesh* mesh = godot::CylinderMesh::_new();

    node->setStartAttributes(
        mesh, avi::connection::SentElementType::CYLINDER, std::move(name), layer);

    node->setShape(godot::CylinderShape::_new());

    return node;
}
