#include "ElementNode.h"

#include <ArmarXCore/core/exceptions/local/UnexpectedEnumValueException.h>

#include <RobotAPI/components/ArViz/ArVizStorage.h>

#include <MainLoop.hpp>
#include <Material.hpp>
#include <arviz_godot/lib/avi/core/Constants.h>

avi::scene::ElementNode::ElementNode()
{
}

void
avi::scene::ElementNode::_register_methods()
{
    register_method("_notification", &ElementNode::_notification);
}

void
avi::scene::ElementNode::_notification(int what)
{
    if (what == godot::MainLoop::NOTIFICATION_PREDELETE && deselectCallback.has_value())
    {
        (*deselectCallback)();
    }
}

avi::connection::SentElementType
avi::scene::ElementNode::getType()
{
    return type;
}

std::string
avi::scene::ElementNode::getName()
{
    return name;
}

avi::core::Layer*
avi::scene::ElementNode::getLayer()
{
    return layer;
}

int
avi::scene::ElementNode::getFlag()
{
    return interactionFlags;
}

bool
avi::scene::ElementNode::isSelectable()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::SELECT;
}

bool
avi::scene::ElementNode::hasContextMenu()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::CONTEXT_MENU;
}

bool
avi::scene::ElementNode::translationX()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::TRANSLATION_X;
}

bool
avi::scene::ElementNode::translationY()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::TRANSLATION_Y;
}

bool
avi::scene::ElementNode::translationZ()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::TRANSLATION_Z;
}

bool
avi::scene::ElementNode::translationLocal()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::TRANSLATION_LOCAL;
}

bool
avi::scene::ElementNode::rotationX()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::ROTATION_X;
}

bool
avi::scene::ElementNode::rotationY()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::ROTATION_Y;
}

bool
avi::scene::ElementNode::rotationZ()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::ROTATION_Z;
}

bool
avi::scene::ElementNode::rotationLocal()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::ROTATION_LOCAL;
}

bool
avi::scene::ElementNode::scalingX()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::SCALING_X;
}

bool
avi::scene::ElementNode::scalingY()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::SCALING_Y;
}

bool
avi::scene::ElementNode::scalingZ()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::SCALING_Z;
}

bool
avi::scene::ElementNode::scalingLocal()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::SCALING_LOCAL;
}

bool
avi::scene::ElementNode::transformHide()
{
    return interactionFlags & armarx::viz::data::InteractionEnableFlags::TRANSFORM_HIDE;
}

std::vector<std::string>
avi::scene::ElementNode::getInteractions()
{
    return interactions;
}

void
avi::scene::ElementNode::setType(avi::connection::SentElementType type)
{
    this->type = type;
}

void
avi::scene::ElementNode::setName(const std::string& name)
{
    this->name = name;
    this->set_name(name.c_str());
}

void
avi::scene::ElementNode::setLayer(avi::core::Layer* layer)
{
    this->layer = layer;
}

void
avi::scene::ElementNode::setInteractions(std::vector<std::string> interactions)
{
    this->interactions = interactions;
}

void
avi::scene::ElementNode::setInteractionFlag(int flag)
{
    interactionFlags = flag;
}

void
avi::scene::ElementNode::_init()
{
}

bool
avi::scene::ElementNode::isManipulable()
{
    bool isManipulable = false;
    isManipulable = isManipulable || translationX() || translationY() || translationZ();
    isManipulable = isManipulable || rotationX() || rotationY() || rotationZ();
    isManipulable = isManipulable || scalingX() || scalingY() || scalingZ();
    return isManipulable;
}

void
avi::scene::ElementNode::deselect()
{
    if (ghostElement)
    {
        ghostElement->queue_free();
        ghostElement = nullptr;
    }
}

void
avi::scene::ElementNode::setMeshVisible(bool visible)
{
    // This is needed for ghost elements, because when setting the element node invisible, you can't see the ghost element as well.

    std::vector<godot::Spatial*> vec = getSpatial();
    for (auto spatial : vec)
    {
        spatial->set_visible(visible);
    }
}

void
avi::scene::ElementNode::createGhostElement(godot::Transform transform)
{
    if (!ghostElement)
    {
        std::vector<godot::Spatial*> vec = getSpatial();
        auto* ghost = godot::Spatial::_new();
        for (auto spatial : vec)
        {
            ghost->add_child(spatial->duplicate(godot::Node::DUPLICATE_SIGNALS |
                                                godot::Node::DUPLICATE_GROUPS |
                                                godot::Node::DUPLICATE_SCRIPTS));
        }

        this->add_child(ghost);
        ghostElement = ghost;
    }

    setMeshVisible(
        !(this->interactionFlags & armarx::viz::data::InteractionEnableFlags::TRANSFORM_HIDE));

    godot::Transform scaler;
    scaler.scale(godot::Vector3::ONE * (1 / avi::core::GODOT_TO_ARMARX_CONVERSION_FACTOR));

    ghostElement->set_transform(transform * scaler);
}

void
avi::scene::ElementNode::setDeselectCallback(std::function<void()> callback)
{
    this->deselectCallback = callback;
}

void
avi::scene::ElementNode::update(IceInternal::Handle<armarx::viz::data::Element> elementInfo)
{
    throw std::logic_error("Function not implemented");
}

std::vector<godot::Spatial*>
avi::scene::ElementNode::getSpatial()
{
    throw std::logic_error("Function not implemented");
}

godot::AABB
avi::scene::ElementNode::getVolume()
{
    throw std::logic_error("Function not implemented");
}

godot::Transform
avi::scene::ElementNode::getTransform()
{
    throw std::logic_error("Function not implemented");
}

avi::scene::ModelDrawStyle::ModelDrawStyle() : armarXCodedValue(-1)
{
}

avi::scene::ModelDrawStyle::ModelDrawStyle(int drawStyle) : armarXCodedValue(drawStyle)
{
    if ((drawStyle & armarx::viz::data::ModelDrawStyle::COLLISION) != 0)
    {
        model = COLLISION;
    }
    else
    {
        model = VISUALIZATION;
    }

    overrideColor = (drawStyle & armarx::viz::data::ModelDrawStyle::OVERRIDE_COLOR) != 0;
}

avi::scene::ModelDrawStyle::Model
avi::scene::ModelDrawStyle::getModel() const
{
    return model;
}

bool
avi::scene::ModelDrawStyle::isVisualization() const
{
    return model == VISUALIZATION;
}

bool
avi::scene::ModelDrawStyle::isCollision() const
{
    return model == COLLISION;
}

bool
avi::scene::ModelDrawStyle::isColorOverriden() const
{
    return overrideColor;
}

avi::scene::DrawStyle
avi::scene::ModelDrawStyle::getSceneDrawStyle()
{
    switch (model)
    {
        case VISUALIZATION:
            return avi::scene::DrawStyle::VISUALIZATION;
        case COLLISION:
            return avi::scene::DrawStyle::COLLISION;
        default:
            ARMARX_UNEXPECTED_ENUM_VALUE(ModelDrawStyle::Model, model);
    }
}

bool
avi::scene::ModelDrawStyle::operator==(const avi::scene::ModelDrawStyle& other) const
{
    return this->armarXCodedValue == other.armarXCodedValue;
}

bool
avi::scene::ModelDrawStyle::operator!=(const avi::scene::ModelDrawStyle& other) const
{
    return not operator==(other);
}
