#include "EllipsoidElementNode.h"

#include <SphereShape.hpp>
#include <arviz_godot/lib/avi/core/Utilities.h>
#include <arviz_godot/lib/avi/scene/Converter.h>

void
avi::scene::EllipsoidElementNode::_register_methods()
{
    register_method("_ready", &EllipsoidElementNode::_ready);
}

void
avi::scene::EllipsoidElementNode::_ready()
{
}

void
avi::scene::EllipsoidElementNode::_init()
{
}

void
avi::scene::EllipsoidElementNode::update(
    IceInternal::Handle<armarx::viz::data::Element> elementInfo)
{
    auto ellipsoid =
        IceInternal::Handle<armarx::viz::data::ElementEllipsoid>::dynamicCast(elementInfo);

    godot::Vector3 newScale = {
        ellipsoid->axisLengths.e0, ellipsoid->axisLengths.e1, ellipsoid->axisLengths.e2};

    if (!avi::core::utilities::is_equal_approx(scale, newScale))
    {
        scale = newScale;
        getMeshInstance()->set_scale(scale);
        getCollisionShape()->set_scale(scale);
    }

    updateAttributes(ellipsoid->interaction.enableFlags,
                     ellipsoid->interaction.contextMenuOptions,
                     avi::scene::Converter::getPosition(ellipsoid->pose),
                     avi::scene::Converter::getScale(ellipsoid->scale),
                     avi::scene::Converter::getRotation(ellipsoid->pose),
                     avi::scene::Converter::getColor(ellipsoid->color));
}

avi::scene::EllipsoidElementNode*
avi::scene::EllipsoidElementNode::newEllipsoidElementNode(std::string name, avi::core::Layer* layer)
{
    EllipsoidElementNode* node = EllipsoidElementNode::_new();
    godot::SphereMesh* mesh = godot::SphereMesh::_new();

    mesh->set_height(2);
    mesh->set_radius(1);

    node->setStartAttributes(mesh, avi::connection::SentElementType::ELLIPSOID, name, layer);
    node->setShape(godot::SphereShape::_new());

    return node;
}
