#include "LayerNode.h"

bool
avi::scene::LayerName::operator==(const avi::scene::LayerName& other) const
{
    return std::tie(component, name) == std::tie(other.component, other.name);
}

bool
avi::scene::LayerName::operator<(const avi::scene::LayerName& other) const
{
    return std::tie(component, name) < std::tie(other.component, other.name);
}

void
avi::scene::LayerNode::_register_methods()
{
    register_method("_notification", &LayerNode::_notification);
}

void
avi::scene::LayerNode::_init()
{
    visible = true;
}

bool
avi::scene::LayerNode::isVisible() const
{
    return visible;
}

void
avi::scene::LayerNode::setVisibility(bool isVisible)
{
    if (visible == isVisible)
        return;

    if(not isVisible) {
        for (auto& [_, element] : elements)
        {
            this->remove_child(element);
        }
    }
    else
    {
        for (auto& [_, element] : elements)
        {
            this->add_child(element);
        }
    }

    this->set_visible(isVisible);
    visible = isVisible;
}

void
avi::scene::LayerNode::_notification(int what)
{
    if (what == godot::Object::NOTIFICATION_PREDELETE)
    {
        if (not isVisible())
        {
            for (auto& [_, element] : elements)
            {
                if (not element->is_queued_for_deletion())
                {
                    element->queue_free();
                }
            }
        }
    }
}

void
avi::scene::LayerNode::setName(const std::string& component, const std::string& name)
{
    this->name = name;
    this->component = component;

    std::string fullName = component + " -- " + name;
    this->set_name(fullName.c_str());
}

const std::string&
avi::scene::LayerNode::getName()
{
    return name;
}

const std::string&
avi::scene::LayerNode::getComponent()
{
    return component;
}

void
avi::scene::LayerNode::addElement(avi::scene::ElementNode* node)
{
    add_child(node);
    elements.insert(std::pair<std::string, avi::scene::ElementNode*>(node->getName(), node));
}

avi::scene::ElementNode*
avi::scene::LayerNode::getElement(std::string name)
{
    if (elements.find(name) != elements.end())
    {
        return elements.find(name)->second;
    }
    else
    {
        return nullptr;
    }
}

void
avi::scene::LayerNode::removeElement(std::string name)
{
    if (elements.find(name) != elements.end())
    {
        if(not this->is_a_parent_of(elements.at(name))) {
            this->add_child(elements.at(name));
        }

        remove_child(elements.at(name));
        elements.erase(name);
    }
}

void
avi::scene::LayerNode::saveRemove() {
    for (auto& [_, element] : elements)
    {
        if(not this->is_a_parent_of(element)) {
            this->add_child(element);
        }
    }

    this->queue_free();
}


godot::AABB
avi::scene::LayerNode::getAABB()
{
    bool noBaseAABB = true;
    godot::AABB aabb;

    for (const auto& [_, element] : elements)
    {
        if (noBaseAABB)
        {
            aabb = element->getVolume();
            noBaseAABB = false;
        }
        else
        {
            aabb = aabb.merge(element->getVolume());
        }
    }

    return aabb;
}