#pragma once

#include "MeshElementNode.h"

namespace godot
{
    class SphereMesh;
    class CylinderMesh;
    class SpatialMaterial;
    class CollisionShape;
} // namespace godot

namespace avi::scene
{
    class LineElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(LineElementNode, ElementNode)

    public:
        ~LineElementNode() override = default;

        static void _register_methods();

        void _ready();

        void _init();

        static LineElementNode* newLineElementNode(std::string name, avi::core::Layer* layer);

        void setStartAttributes(std::string name, avi::core::Layer* layer);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color);

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;
        std::vector<godot::Spatial*> getSpatial() override;

    private:
        void updateRadius(float newRadius);
        void updateLine(godot::Vector3 from, godot::Vector3 to);

        void setPosition(godot::Vector3 position);
        void setRotation(godot::Quat rotation);
        void setScale(godot::Vector3 scale);
        void setColor(godot::Color color);

    private:
        float radius = 0.0;
        godot::Vector3 a = godot::Vector3::ZERO;
        godot::Vector3 b = godot::Vector3::ZERO;

        godot::StaticBody* staticBody;
        godot::CollisionShape* collisionLine;
        godot::SpatialMaterial* material;

        godot::MeshInstance* start;
        godot::MeshInstance* end;
        godot::MeshInstance* line;
    };
} // namespace avi::scene
