#pragma once

#include "ElementNode.h"
#include <CollisionShape.hpp>
#include <Mesh.hpp>
#include <MeshInstance.hpp>
#include <StaticBody.hpp>
#include <arviz_godot/lib/avi/core/Element.h>

namespace avi::scene
{
    class MeshElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(MeshElementNode, ElementNode)

    public:
        MeshElementNode();
        ~MeshElementNode() override = default;

        void _init();

        static void _register_methods();

        void _ready();

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void setPosition(godot::Vector3 position);

        void setScale(godot::Vector3 scale);

        void setRotation(godot::Quat rotation);

        void setColor(godot::Color color);

        void setMesh(godot::Mesh* mesh);

        void setShape(godot::Mesh* mesh);
        void setShape(godot::Shape* shape);

        godot::Vector3 getPosition();

        godot::Vector3 getRotation();

        godot::Vector3 getScale();

        godot::MeshInstance* getMeshInstance();
        godot::CollisionShape* getCollisionShape();

        void setStartAttributes(godot::Mesh* mesh,
                                avi::connection::SentElementType type,
                                std::string name,
                                avi::core::Layer* layer);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color);

        godot::Mesh* getMesh();

        void useVertexColor(bool isUsed);

        void setTwoSided();

    private:
        godot::StaticBody* staticBody;
        godot::CollisionShape* collisionShape;
        godot::MeshInstance* meshInstance;

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };
} // namespace avi::scene
