#pragma once

#include "ElementNode.h"
#include <CollisionShape.hpp>
#include <Mesh.hpp>
#include <MeshInstance.hpp>
#include <Shape.hpp>
#include <StaticBody.hpp>

namespace avi::scene
{
    class MultiMeshElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(MultiMeshElementNode, ElementNode)

    public:
        MultiMeshElementNode();
        ~MultiMeshElementNode() override = default;

        void _init();

        static void _register_methods();

        void _ready();

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void setPosition(godot::Vector3 position, size_t index);

        void setScale(godot::Vector3 scale, size_t index);

        void setRotation(godot::Quat rotation, size_t index);

        void setColor(godot::Color color, size_t index);

        void setMesh(std::vector<godot::Mesh*> meshes);

        void setShape();

        std::vector<godot::Vector3> getPosition();

        std::vector<godot::Vector3> getRotation();

        std::vector<godot::Vector3> getScale();

        void setStartAttributes(std::vector<godot::Mesh*> meshes,
                                avi::connection::SentElementType type,
                                std::string name,
                                avi::core::Layer* layer);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color);

        godot::Mesh* getMesh(int index);

        void setTwoSided(int index);

        void setVisibility(int index, bool visible);

    private:
        std::vector<godot::MeshInstance*> meshInstances;
        godot::StaticBody* staticBody;
        std::vector<godot::CollisionShape*> collisionShapes;

        bool shapeChecked = false;

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };
} // namespace avi::scene
