#pragma once

#include <vector>

#include "ElementNode.h"
#include <CollisionShape.hpp>
#include <Mesh.hpp>
#include <MeshInstance.hpp>
#include <Shape.hpp>
#include <StaticBody.hpp>
#include <Texture.hpp>
#include <arviz_godot/lib/avi/core/ModelLoader.h>

namespace avi::scene
{
    class ObjectElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(ObjectElementNode, ElementNode)

    public:
        ObjectElementNode();
        ~ObjectElementNode() override = default;

        void _init();

        static ObjectElementNode* newObjectElementNode(avi::core::ModelLoader* loader,
                                                       avi::connection::SentElementType type,
                                                       std::string name,
                                                       avi::core::Layer* layer);

        static void _register_methods();

        void _ready();

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        void setPosition(godot::Vector3 position);

        void setScale(godot::Vector3 scale);

        void setRotation(godot::Quat rotation);

        godot::Vector3 getPosition();

        godot::Vector3 getRotation();

        godot::Vector3 getScale();

        void setShape(VirtualRobot::ManipulationObjectPtr visualization);

        void setStartAttributes(avi::connection::SentElementType type,
                                std::string name,
                                avi::core::Layer* layer);

        void updateAttributes(std::string fileName,
                              int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color,
                              ModelDrawStyle drawStyle);

        void setOverrideColor(godot::Color color, ModelDrawStyle drawStyle);

    private:
        avi::core::ModelLoader* loader{nullptr};

        std::string fileName;

        godot::StaticBody* staticBody{nullptr};

        godot::MeshInstance* meshInstance{nullptr};
        godot::CollisionShape* collisionShape{nullptr};

        ModelDrawStyle currentDrawStyle{};
        godot::Color currentColor{};

        bool shapeChecked = false;

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };
} // namespace avi::scene
