#pragma once

#include "ElementNode.h"

namespace godot
{
    class StaticBody;
    class CollisionShape;
    class SpatialMaterial;
    class MultiMesh;
} // namespace godot

namespace avi::scene
{
    class PathElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(PathElementNode, ElementNode)

    public:
        ~PathElementNode() override = default;

        static void _register_methods();

        void _ready();

        void _init();

        static avi::scene::PathElementNode* newPathElementNode(std::string name,
                                                               avi::core::Layer* layer);

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        void setStartAttributes(std::string name, avi::core::Layer* layer);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color);

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;
        std::vector<godot::Spatial*> getSpatial() override;

    private:
        void buildPath();

        void buildLines();
        void buildSpheres();

        void setPosition(godot::Vector3 position);
        void setRotation(godot::Quat rotation);
        void setScale(godot::Vector3 scale);
        void setColor(godot::Color color);

    private:
        float radius = 0.0;
        std::vector<godot::Vector3> points = {};

        std::vector<godot::Spatial*> spatials = {};

        godot::StaticBody* staticBody;
        godot::SpatialMaterial* material;

        godot::MultiMesh* meshLine;
        godot::MultiMesh* meshSphere;

        godot::PoolRealArray lineData;
        godot::PoolRealArray sphereData;

        godot::Vector3 maxPoint;
        godot::Vector3 minPoint;

        std::vector<godot::CollisionShape*> shapes = {};
    };
} // namespace avi::scene
