#pragma once

#include "CameraControlScheme.h"
#include <Godot.hpp>
#include <Spatial.hpp>
#include <arviz_godot/lib/avi/scene/ViewCamera.h>
#include <arviz_godot/lib/avi/scene/VisualizedScene.h>

namespace avi::scene
{
    class PlayfulControlScheme : public godot::Spatial, public avi::scene::CameraControlScheme
    {
        GODOT_CLASS(PlayfulControlScheme, Spatial)

    public:
        ~PlayfulControlScheme() override = default;

        static void _register_methods();

        void _init();

        void _ready();

        void _process(float delta);

        void _unhandled_input(godot::InputEvent* event);

    public:
        void setup(avi::core::SettingsProvider& settings,
                   avi::scene::VisualizedScene& scene,
                   bool gravity);

        void takeOver(avi::scene::ViewCamera* viewCamera, godot::Transform head) override;

        godot::Transform release() override;

        void reset() override;

        void focus(godot::AABB box) override;

        void focus(godot::Vector3 point, avi::core::Element* target) override;

        void placeOnAxis(godot::Vector3 fromVector) override;

        void invertAxis(godot::Vector3 fromVector) override;

        void placeOnAxisNumpad(Axis axis, AxisSign sign) override;

        void rotateFixed(RotationDirection direction, HotkeyRotation hotkey) override;

        godot::Basis getRotationBasis() override;

        void setupControlAdvertisements(avi::core::ControlAdvertiser& advertiser) override;

        void updateControlAdvertisements();

    private:
        bool isActive();

        void startTravel(godot::Transform target, float duration);

        void doTravel(float delta);

    private:
        struct Travel
        {
            godot::Transform startTransform{};
            godot::Transform endTransform{};
            godot::Transform endTransformHead{};

            float duration{};
            float remainingTime{};
        };

        Travel travel{};
        bool isTraveling{false};

        bool gravity = false;

        godot::Spatial* attachment;
        godot::Spatial* head;
        avi::scene::ViewCamera* viewCamera;

        float cameraStartSize = 1.0f;
        float cameraEndSize = 1.0f;
        bool updateSize = false;

        std::vector<bool> inputState;
        std::vector<const char*> inputKeys;

        enum InputStateIndex
        {
            INPUT_FORWARD,
            INPUT_BACKWARD,
            INPUT_LEFT,
            INPUT_RIGHT,
            INPUT_UP,
            INPUT_DOWN,
            INPUT_ROLL_LEFT,
            INPUT_ROLL_RIGHT,
            MAX_INPUT_INDEX
        };

        bool switchX = false;
        bool switchY = false;
        bool switchZ = false;

        bool rotatePressed = false;
        bool continuousEnabled = true;
        godot::Vector2 pressedRotation;

        avi::core::SettingsProvider* settings;
        avi::scene::VisualizedScene* scene;

        godot::Vector2 mouseDelta;
        float scrollValue;

        core::ControlAdvertiser* advertiser;
        size_t adID;
        size_t adIDOrthographic;
    };
} // namespace avi::scene
