#pragma once

#include <RobotAPI/interface/ArViz/Elements.h>

#include <ArrayMesh.hpp>
#include <CollisionShape.hpp>
#include <Mesh.hpp>
#include <MeshInstance.hpp>
#include <SpatialMaterial.hpp>
#include <StaticBody.hpp>
#include <Texture.hpp>
#include <arviz_godot/lib/avi/scene/ElementNode.h>
#include <arviz_godot/lib/avi/scene/GraphicsController.h>

namespace avi::scene
{

    class PointCloudElementNode : public avi::scene::ElementNode
    {
        GODOT_CLASS(PointCloudElementNode, ElementNode)

    public:
        PointCloudElementNode();
        ~PointCloudElementNode() override = default;

        void _init();

        godot::AABB getVolume() override;
        godot::Transform getTransform() override;

        void update(IceInternal::Handle<armarx::viz::data::Element> elementInfo) override;

        static void _register_methods();

        void _ready();

        void drawPointCloud(ulong pointCloudSize, armarx::viz::data::ColoredPoint const* pointPtr);

        static PointCloudElementNode*
        newPointCloudElementNode(avi::connection::SentElementType type,
                                 std::string name,
                                 avi::core::Layer* layer);

        void setPosition(godot::Vector3 position);

        void setScale(godot::Vector3 scale);

        void setColor(godot::Color color);

        void setRotation(godot::Quat rotation);

        void setShape();

        godot::Vector3 getPosition();

        godot::Vector3 getScale();

        void setStartAttributes(avi::connection::SentElementType type,
                                std::string name,
                                avi::core::Layer* layer);

        void updateAttributes(int flag,
                              std::vector<std::string> interactions,
                              godot::Vector3 position,
                              godot::Vector3 scale,
                              godot::Quat rotation,
                              godot::Color color);

    private:
        godot::MeshInstance* meshInstance;
        godot::ArrayMesh* mesh;

        godot::PoolVector3Array pointData{};
        godot::PoolColorArray colorData{};

        godot::Ref<godot::SpatialMaterial> material;
        godot::Vector3 position;
        godot::Vector3 scale;
        godot::Color color;

        godot::StaticBody* staticBody;
        godot::CollisionShape* collisionShape;

        godot::Vector3 minPoint{};
        godot::Vector3 maxPoint{};

        bool shapeExists = false;
        godot::AABB shapeVolume;

    protected:
        std::vector<godot::Spatial*> getSpatial() override;
    };
} // namespace avi::scene
