#include "PoseElementNode.h"

#include <iostream>

#include <CubeMesh.hpp>
#include <Font.hpp>
#include <ResourceLoader.hpp>
#include <arviz_godot/lib/avi/core/Constants.h>
#include <arviz_godot/lib/avi/scene/Converter.h>

godot::Mesh* avi::scene::PoseElementNode::mesh = {};

avi::scene::PoseElementNode::PoseElementNode() = default;

godot::AABB
avi::scene::PoseElementNode::getVolume()
{
    godot::AABB AABB = poseMeshInstance->get_aabb();
    AABB.position = staticBody->get_transform().origin;
    AABB.size = AABB.size * staticBody->get_scale();

    return AABB;
}

void
avi::scene::PoseElementNode::_register_methods()
{
}

void
avi::scene::PoseElementNode::setPosition(godot::Vector3 position)
{
    auto transform = staticBody->get_transform();
    transform.set_origin(position);
    staticBody->set_transform(transform);

    label->set_translation({5, 5, 2.5});
}

void
avi::scene::PoseElementNode::setScale(godot::Vector3 scale)
{
    staticBody->set_scale(scale);
}

void
avi::scene::PoseElementNode::setRotation(godot::Quat rotation)
{
    godot::Transform quaternionTransform = godot::Transform(rotation);
    quaternionTransform.origin = staticBody->get_transform().origin;

    staticBody->set_transform(quaternionTransform);
}

void
avi::scene::PoseElementNode::setColor(godot::Color color)
{
}

godot::Vector3
avi::scene::PoseElementNode::getPosition()
{
    return staticBody->get_translation();
}

godot::Vector3
avi::scene::PoseElementNode::getRotation()
{
    return staticBody->get_rotation();
}

godot::Vector3
avi::scene::PoseElementNode::getScale()
{
    return staticBody->get_scale();
}

avi::scene::PoseElementNode*
avi::scene::PoseElementNode::newPoseElementNode(avi::connection::SentElementType type,
                                                std::string name,
                                                avi::core::Layer* layer)
{
    PoseElementNode* node = newPoseElementNode();
    node->setStartAttributes(type, name, layer);

    return node;
}

avi::scene::PoseElementNode*
avi::scene::PoseElementNode::newPoseElementNode()
{
    godot::ResourceLoader* loader = godot::ResourceLoader::get_singleton();
    godot::Ref<godot::Font> font = loader->load("res://fonts/font.tres");

    PoseElementNode* node = PoseElementNode::_new();

    node->staticBody = godot::StaticBody::_new();
    node->collisionShape = godot::CollisionShape::_new();
    node->poseMeshInstance = godot::MeshInstance::_new();

    node->label = godot::Label3D::_new();
    node->label->set_font(font);
    node->label->set_pixel_size(node->textDownscaling);
    node->label->set_modulate(godot::Color());
    //node->label->set_offset({0, -5.0f / node->textDownscaling});
    node->label->set_horizontal_alignment(godot::Label3D::ALIGN_LEFT);
    node->label->set_vertical_alignment(godot::Label3D::VALIGN_BOTTOM);

    node->staticBody->add_child(node->collisionShape);
    node->staticBody->add_child(node->poseMeshInstance);
    node->staticBody->add_child(node->label);

    node->add_child(node->staticBody);

    return node;
}

void
avi::scene::PoseElementNode::setStartAttributes(avi::connection::SentElementType type,
                                                std::string name,
                                                avi::core::Layer* layer)
{
    setType(type);
    setName(name);
    setLayer(layer);

    if (mesh == nullptr)
    {
        godot::ResourceLoader* loader = godot::ResourceLoader::get_singleton();
        mesh = godot::Ref<godot::Mesh>(loader->load("res://models/pose.obj")).ptr();
    }

    poseMeshInstance->set_mesh(mesh);

    label->set_text(name.c_str());
}

void
avi::scene::PoseElementNode::updateAttributes(int flag,
                                              std::vector<std::string> interactions,
                                              godot::Vector3 position,
                                              godot::Vector3 scale,
                                              godot::Quat rotation,
                                              godot::Color color)
{
    setInteractionFlag(flag);

    if (!shapeChecked)
    {
        setShape();
        shapeChecked = true;
    }

    setInteractions(interactions);
    setPosition(position);
    setColor(color);
    setRotation(rotation);
    setScale(scale);
}

void
avi::scene::PoseElementNode::_init()
{
}

void
avi::scene::PoseElementNode::update(IceInternal::Handle<armarx::viz::data::Element> elementInfo)
{
    auto pose = IceInternal::Handle<armarx::viz::data::ElementPose>::dynamicCast(elementInfo);
    updateAttributes(pose->interaction.enableFlags,
                     pose->interaction.contextMenuOptions,
                     avi::scene::Converter::getPosition(pose->pose),
                     avi::scene::Converter::getScale(pose->scale),
                     avi::scene::Converter::getRotation(pose->pose),
                     avi::scene::Converter::getColor(pose->color));
}

godot::Transform
avi::scene::PoseElementNode::getTransform()
{
    godot::Transform transform = staticBody->get_transform();
    transform.basis.scale(godot::Vector3::ONE * avi::core::GODOT_TO_ARMARX_CONVERSION_FACTOR);

    return transform;
}

void
avi::scene::PoseElementNode::setShape()
{
    godot::Ref<godot::Shape> shape = mesh->create_trimesh_shape();
    this->collisionShape->set_shape(shape);
}

std::vector<godot::Spatial*>
avi::scene::PoseElementNode::getSpatial()
{
    std::vector<godot::Spatial*> vec;
    vec.push_back(poseMeshInstance);
    return vec;
}
