#include "SphereElementNode.h"

#include <SphereMesh.hpp>
#include <SphereShape.hpp>
#include <arviz_godot/lib/avi/scene/Converter.h>

void
avi::scene::SphereElementNode::_register_methods()
{
    register_method("_ready", &SphereElementNode::_ready);
}

void
avi::scene::SphereElementNode::_ready()
{
}

void
avi::scene::SphereElementNode::_init()
{
}

void
avi::scene::SphereElementNode::update(IceInternal::Handle<armarx::viz::data::Element> elementInfo)
{
    auto sphere = IceInternal::Handle<armarx::viz::data::ElementSphere>::dynamicCast(elementInfo);

    float newRadius = sphere->radius;

    if (!godot::Math::is_equal_approx(radius, newRadius))
    {
        radius = newRadius;

        getMeshInstance()->set_scale({radius, radius, radius});
        getCollisionShape()->set_scale({radius, radius, radius});
    }


    updateAttributes(sphere->interaction.enableFlags,
                     sphere->interaction.contextMenuOptions,
                     avi::scene::Converter::getPosition(sphere->pose),
                     avi::scene::Converter::getScale(sphere->scale),
                     avi::scene::Converter::getRotation(sphere->pose),
                     avi::scene::Converter::getColor(sphere->color));
}

avi::scene::SphereElementNode*
avi::scene::SphereElementNode::newSphereElementNode(std::string name, avi::core::Layer* layer)
{
    SphereElementNode* node = SphereElementNode::_new();
    godot::Mesh* mesh = godot::SphereMesh::_new();

    node->setStartAttributes(mesh, avi::connection::SentElementType::SPHERE, name, layer);
    node->setShape(godot::SphereShape::_new());

    return node;
}
