#pragma once

#include <Godot.hpp>
#include <Spatial.hpp>
#include <arviz_godot/lib/avi/scene/CameraControlScheme.h>
#include <arviz_godot/lib/avi/scene/VisualizedScene.h>

namespace avi::scene
{
    class ViewCamera;

    class TechnicalControlScheme : public godot::Spatial, public avi::scene::CameraControlScheme
    {
        GODOT_CLASS(TechnicalControlScheme, Spatial)

    public:
        ~TechnicalControlScheme() override = default;
        static void _register_methods();

        void _init();

        void _ready();

        void _unhandled_input(godot::InputEvent* event);

        void _process(float delta);

    public:
        void setup(avi::core::SettingsProvider& settings,
                   avi::scene::VisualizedScene& scene,
                   bool gravity);

        void takeOver(avi::scene::ViewCamera* viewCamera, godot::Transform head) override;

        godot::Transform release() override;

        void reset() override;

        void focus(godot::AABB box) override;

        void focus(godot::Vector3 point, avi::core::Element* target) override;

        godot::Basis getRotationBasis() override;

        void placeOnAxis(godot::Vector3 fromVector) override;

        void invertAxis(godot::Vector3 fromVector) override;

        void setupControlAdvertisements(avi::core::ControlAdvertiser& advertiser) override;

    private:
        bool isActive();

        void orbit(godot::Vector2 rotation, float deltaTime);

        void rotateFixed(RotationDirection direction, HotkeyRotation hotkey) override;

        void placeOnAxisNumpad(Axis axis, AxisSign sign) override;

        void move(godot::Vector2 movement, float deltaTime);

        void zoom(float delta, float deltaTime);

        static float getClampedZoom(float zoom);

        void zoomOrthographic(float delta, float deltaTime);

        void roll(float delta, float deltaTime);

        void startTravel(godot::Transform anchor, float zoom, float duration);

        void doTravel(float deltaTime);

    private:
        struct Travel
        {
            godot::Transform startTransform{};
            godot::Transform endTransform{};
            godot::Transform endTransformHead{};

            float startZoom{};
            float endZoom{};

            float duration{};
            float remainingTime{};
        };

        godot::Vector2 mouseDelta;
        float wheelDelta;
        float smoothWheelDelta;

        float cameraStartSize = 1.0f;
        float cameraEndSize = 1.0f;
        bool updateSize = false;

        float zoomValue;
        float zoomDistance;

        bool gravity = false;

        bool switchX = false;
        bool switchY = false;
        bool switchZ = false;

        bool rotatePressed = false;
        bool continuousEnabled = true;
        godot::Vector2 pressedRotation;

        godot::Spatial* anchor;
        godot::Spatial* head;
        godot::Spatial* zoomer;
        godot::Spatial* attachment;

        avi::scene::ViewCamera* viewCamera;

        avi::core::SettingsProvider* settings;
        avi::scene::VisualizedScene* scene;

        Travel travel;
        bool isTraveling{};

        core::ControlAdvertiser* advertiser;
        size_t adID;
    };
} // namespace avi::scene
