#include "ViewCamera.h"

#include <Dictionary.hpp>
#include <ImageTexture.hpp>
#include <Input.hpp>
#include <InputEventMouseButton.hpp>
#include <Light.hpp>
#include <PackedScene.hpp>
#include <ResourceLoader.hpp>
#include <ViewportTexture.hpp>
#include <World.hpp>
#include <Node.hpp>
#include <SceneTree.hpp>
#include <arviz_godot/lib/avi/scene/PlayfulControlScheme.h>
#include <arviz_godot/lib/avi/scene/TechnicalControlScheme.h>

void
avi::scene::ViewCamera::_register_methods()
{
}

void
avi::scene::ViewCamera::_init()
{
}

void
avi::scene::ViewCamera::setup(avi::scene::VisualizedScene& scene,
                              avi::core::EventSubscriptionSystem& events,
                              avi::core::SettingsProvider& settings,
                              avi::scene::GraphicsController& graphics)
{

    this->scene = &scene;

    events.subscribe("ORBIT_NEGATIVE_HORIZONTAL",
                     [this]() -> void {
                         currentScheme->rotateFixed(CameraControlScheme::NEGATIVE_HORIZONTAL,
                                                    CameraControlScheme::continuous);
                     });
    events.subscribe("ORBIT_POSITIVE_HORIZONTAL",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::POSITIVE_HORIZONTAL,
                                                    CameraControlScheme::continuous);
                     });
    events.subscribe("ORBIT_NEGATIVE_VERTICAL",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::NEGATIVE_VERTICAL,
                                                    CameraControlScheme::continuous);
                     });
    events.subscribe("ORBIT_POSITIVE_VERTICAL",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::POSITIVE_VERTICAL,
                                                    CameraControlScheme::continuous);
                     });


    events.subscribe("ORBIT_NEGATIVE_HORIZONTAL_SHIFT",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::NEGATIVE_HORIZONTAL,
                                                    CameraControlScheme::fixed);
                     });
    events.subscribe("ORBIT_POSITIVE_HORIZONTAL_SHIFT",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::POSITIVE_HORIZONTAL,
                                                    CameraControlScheme::fixed);
                     });
    events.subscribe("ORBIT_NEGATIVE_VERTICAL_SHIFT",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::NEGATIVE_VERTICAL,
                                                    CameraControlScheme::fixed);
                     });
    events.subscribe("ORBIT_POSITIVE_VERTICAL_SHIFT",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::POSITIVE_VERTICAL,
                                                    CameraControlScheme::fixed);
                     });


    events.subscribe("ORBIT_NEGATIVE_HORIZONTAL_RELEASE",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::NEGATIVE_HORIZONTAL,
                                                    CameraControlScheme::disable);
                     });
    events.subscribe("ORBIT_POSITIVE_HORIZONTAL_RELEASE",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::POSITIVE_HORIZONTAL,
                                                    CameraControlScheme::disable);
                     });
    events.subscribe("ORBIT_NEGATIVE_VERTICAL_RELEASE",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::NEGATIVE_VERTICAL,
                                                    CameraControlScheme::disable);
                     });
    events.subscribe("ORBIT_POSITIVE_VERTICAL_RELEASE",
                     [this]() -> void
                     {
                         currentScheme->rotateFixed(CameraControlScheme::POSITIVE_VERTICAL,
                                                    CameraControlScheme::disable);
                     });


    events.subscribe("PLACE_ON_X_NUMPAD",
                     [this]() -> void {
                         currentScheme->placeOnAxisNumpad(CameraControlScheme::xAxis,
                                                          CameraControlScheme::signSwitch);
                     });
    events.subscribe("PLACE_ON_Y_NUMPAD",
                     [this]() -> void {
                         currentScheme->placeOnAxisNumpad(CameraControlScheme::yAxis,
                                                          CameraControlScheme::signSwitch);
                     });
    events.subscribe("PLACE_ON_Z_NUMPAD",
                     [this]() -> void {
                         currentScheme->placeOnAxisNumpad(CameraControlScheme::zAxis,
                                                          CameraControlScheme::signSwitch);
                     });

    events.subscribe("VIEW_CAMERA_SWITCH_PROJECTION",
                     [this]() -> void { this->switchProjection(); });
    events.subscribe("VIEW_CAMERA_RESET", [this]() -> void { this->reset(); });
    events.subscribe("VIEW_CAMERA_FOCUS_SCENE", [this]() -> void { this->focusScene(); });
    events.subscribe("VIEW_SCHEME_SWITCH", [this]() -> void { this->switchScheme(); });
    events.subscribe("VIEW_CAMERA_FOCUS_POINT", [this]() -> void { this->focusPoint(); });
    events.subscribe("VIEW_CAMERA_LOCK", [this]() -> void { this->toggleLock(); });
    events.subscribe("VIEW_CAMERA_ESCAPE_LOCK",
                     [this]() -> void
                     {
                         if (isMouseLocked())
                             this->toggleLock();
                     });

    godot::ResourceLoader* loader = godot::ResourceLoader::get_singleton();

    godot::Ref<godot::PackedScene> technicalScene =
        loader->load("res://scenes/technical_control_scheme.tscn");
    technicalControlScheme =
        godot::Object::cast_to<avi::scene::TechnicalControlScheme>(technicalScene->instance());
    get_parent()->add_child(technicalControlScheme);

    godot::Ref<godot::PackedScene> playfulScene =
        loader->load("res://scenes/playful_control_scheme.tscn");
    playfulControlScheme =
        godot::Object::cast_to<avi::scene::PlayfulControlScheme>(playfulScene->instance());
    get_parent()->add_child(playfulControlScheme);
    this->playfulControlScheme->setup(settings, scene, settings.cameraGravity());

    get_parent()->remove_child(this);
    technicalControlScheme->takeOver(this, godot::Transform::IDENTITY);
    currentScheme = technicalControlScheme;
    this->technicalControlScheme->setup(settings, scene, settings.cameraGravity());

    graphicsController = &graphics;
    graphics.registerChangeListener(this, [this]() { this->visualizationChanged(); });
    visualizationChanged();

    this->set_znear(0.0001);
    this->set_zfar(300);
}

void
avi::scene::ViewCamera::setupControlAdvertisements(avi::core::ControlAdvertiser& advertiser)
{
    technicalControlScheme->setupControlAdvertisements(advertiser);
    playfulControlScheme->setupControlAdvertisements(advertiser);
}

void
avi::scene::ViewCamera::activate()
{
}

avi::core::ScreenContent
avi::scene::ViewCamera::getCurrentScreenContent()
{
    godot::Ref<godot::Image> data = this->get_viewport()->get_texture()->get_data();
    data->flip_y();

    return {data};
}

godot::Vector2
avi::scene::ViewCamera::getScreenSize()
{
    return get_viewport()->get_size();
}

std::optional<std::pair<avi::core::Element*, godot::Node*>>
avi::scene::ViewCamera::getSelectedElement(godot::Vector2 mouse_position)
{
    bool _;
    return getSelectedElement(mouse_position, _);
}

std::optional<std::pair<avi::core::Element*, godot::Node*>>
avi::scene::ViewCamera::getSelectedElement(godot::Vector2 mouse_position, bool& anyHit)
{
    direct_space_state = get_world()->get_direct_space_state();
    ray_origin_position = this->project_ray_origin(mouse_position);
    ray_end_position = ray_origin_position + this->project_ray_normal(mouse_position) * 1000;
    result_dict = direct_space_state->intersect_ray(ray_origin_position, ray_end_position);

    if (!result_dict.empty())
    {
        anyHit = true;

        godot::Node* node = godot::Object::cast_to<godot::Node>(result_dict["collider"]);
        avi::scene::ElementNode* element =
            godot::Object::cast_to<avi::scene::ElementNode>(node->get_parent());

        if ((node != nullptr) && (element != nullptr))
        {
            std::pair<avi::core::Element*, godot::Node*> result;
            result.first = element;
            result.second = node;
            return result;
        }
    }

    return std::nullopt;
}

std::optional<godot::Vector3>
avi::scene::ViewCamera::screenPointToWorldPoint(godot::Vector2 point)
{
    direct_space_state = get_world()->get_direct_space_state();
    ray_origin_position = this->project_ray_origin(point);
    ray_end_position = ray_origin_position + this->project_ray_normal(point) * 1000;
    result_dict = direct_space_state->intersect_ray(ray_origin_position, ray_end_position);

    if (!result_dict.empty())
    {
        auto* node = godot::Object::cast_to<godot::Spatial>(result_dict["collider"]);

        if (node != nullptr)
        {
            return result_dict["position"];
        }
    }

    return std::nullopt;
}

avi::scene::CameraControlScheme*
avi::scene::ViewCamera::getCurrentScheme()
{
    return currentScheme;
}

void
avi::scene::ViewCamera::toggleLock()
{
    setMouseLocked(!isMouseLocked());
}

void
avi::scene::ViewCamera::focusPoint()
{
    godot::Vector2 mousePosition = this->get_tree()->get_root()->get_viewport()->get_mouse_position();
    auto pos = screenPointToWorldPoint(mousePosition);
    auto target = getSelectedElement(mousePosition);

    if (target.has_value() && target->first && pos.has_value()) {
        currentScheme->focus(pos.value(), target->first);
    }
}

void
avi::scene::ViewCamera::reset()
{
    currentScheme->reset();
}

void
avi::scene::ViewCamera::focusScene()
{
    currentScheme->focus(scene->getSceneVolume());
}

void
avi::scene::ViewCamera::switchScheme()
{
    godot::Transform head = currentScheme->release();

    if (currentScheme == technicalControlScheme)
    {
        currentScheme = playfulControlScheme;
        setMouseLocked(true);
    }
    else
    {
        currentScheme = technicalControlScheme;
    }

    currentScheme->takeOver(this, head);
}

void
avi::scene::ViewCamera::setMouseLocked(bool locked)
{
    godot::Input* input = godot::Input::get_singleton();

    if (isMouseLocked() == locked)
        return;

    if (locked)
    {
        input->set_mouse_mode(godot::Input::MOUSE_MODE_CAPTURED);
    }
    else
    {
        input->set_mouse_mode(godot::Input::MOUSE_MODE_VISIBLE);
    }

    mouseLocked = locked;
}

bool
avi::scene::ViewCamera::isMouseLocked()
{
    return mouseLocked;
}

void
avi::scene::ViewCamera::visualizationChanged()
{
    auto* light = this->get_node<godot::Light>("SpotLight");

    light->set_visible(graphicsController->isHeadlightEnabled());
    light->set_param(godot::Light::PARAM_ENERGY, graphicsController->getHeadlightStrength());
}

void
avi::scene::ViewCamera::setClipPlanes(float near)
{
    const float distance = 4.0f;

    float near_exp = std::log10(near);
    float far_exp = distance + near_exp;
    float far = std::pow(10.0f, far_exp);

    this->set_znear(near);
    this->set_zfar(far);
}

void
avi::scene::ViewCamera::switchProjection()
{
    if (orthogonal)
    {
        set_projection(godot::Camera::PROJECTION_PERSPECTIVE);
    }
    else
    {
        set_projection(godot::Camera::PROJECTION_ORTHOGONAL);
    }

    orthogonal = !orthogonal;
    if (currentScheme == playfulControlScheme)
    {
        playfulControlScheme->updateControlAdvertisements();
    }
}

bool
avi::scene::ViewCamera::isOrthogonal()
{
    return orthogonal;
}

void
avi::scene::ViewCamera::placeOnAxis(godot::Vector3 fromVector)
{
    currentScheme->placeOnAxis(fromVector);
}
void
avi::scene::ViewCamera::invertAxis(godot::Vector3 fromVector)
{
    currentScheme->invertAxis(fromVector);
}
