#include <cstdlib>

#include <arviz_godot/lib/avi/core/OptionFile.h>
#include <catch2/catch_test_macros.hpp>

TEST_CASE("create, save and load an option file", "[OptionFile]")
{
    char tmp[] = "/tmp/tmpdir.XXXXXX";
    auto tmpdir = mkdtemp(tmp);
    REQUIRE(tmpdir != nullptr);

    const std::string name = "actualName";
    const std::filesystem::path temporaryDirectory(tmpdir);
    const std::filesystem::path path = temporaryDirectory / (name + ".opt");

    avi::core::OptionFile optionFile = avi::core::OptionFile::create("initialName");
    optionFile.setName(name);
    optionFile.setIntegerValue("integer", 42);
    optionFile.setDoubleValue("double", 3.14);
    optionFile.setStringValue("string", "hello world");

    optionFile.save(path);

    auto loaded = avi::core::OptionFile::load(path);
    REQUIRE(loaded.has_value());

    avi::core::OptionFile loadedOptionFile = loaded.value();
    CHECK(loadedOptionFile.getName() == name);
    CHECK(loadedOptionFile.getIntegerValue("integer") == 42);
    CHECK(loadedOptionFile.getDoubleValue("double") == 3.14);
    CHECK(loadedOptionFile.getStringValue("string") == "hello world");
}
