#include <cstdlib>

#include <arviz_godot/lib/avi/core/OptionFileReader.h>
#include <arviz_godot/lib/avi/core/OptionFileWriter.h>
#include <catch2/catch_test_macros.hpp>

struct TestContainer
{
    std::string str{};
    int64_t i{};
    double d{};
    bool b{};
    std::filesystem::path p{};

    template <typename T>
    auto
    getOptions()
    {
        std::vector<std::unique_ptr<avi::core::Option<T>>> options;
        options.ADD_OPTION(std::string, str);
        options.ADD_OPTION(int64_t, i);
        options.ADD_OPTION(double, d);
        options.ADD_OPTION(bool, b);
        options.ADD_OPTION(std::filesystem::path, p);

        return options;
    }
};

TEST_CASE("read valid options from option file", "[OptionFileReader]")
{
    TestContainer testContainer;
    avi::core::OptionFileReader<TestContainer> reader(testContainer);

    avi::core::OptionFile file = avi::core::OptionFile::create("test");
    file.setStringValue("str", "test");
    file.setIntegerValue("i", 1);
    file.setDoubleValue("d", 1.0);
    file.setBooleanValue("b", true);
    file.setPathValue("p", "test");

    reader.read(file);

    REQUIRE(testContainer.str == "test");
    REQUIRE(testContainer.i == 1);
    REQUIRE(testContainer.d == 1.0);
    REQUIRE(testContainer.b == true);
    REQUIRE(testContainer.p == "test");
}

TEST_CASE("write valid options to an option file", "[OptionFileWriter]")
{
    TestContainer testContainer;
    testContainer.str = "test";
    testContainer.i = 1;
    testContainer.d = 1.0;
    testContainer.b = true;
    testContainer.p = "test";

    avi::core::OptionFileWriter<TestContainer> writer(testContainer);
    avi::core::OptionFile file = avi::core::OptionFile::create("test");

    writer.write(file);

    REQUIRE(file.getStringValue("str") == "test");
    REQUIRE(file.getIntegerValue("i") == 1);
    REQUIRE(file.getDoubleValue("d") == 1.0);
    REQUIRE(file.getBooleanValue("b") == true);
    REQUIRE(file.getPathValue("p") == "test");
}
