#include <arviz_godot/lib/avi/connection/RevisionBuffer.h>
#include <catch2/catch_test_macros.hpp>

TEST_CASE("add and remove from revision buffer", "[RevisionBuffer]")
{
    avi::connection::RevisionBuffer buffer;
    REQUIRE(buffer.getChangedLayers().size() == 0);

    armarx::viz::data::LayerUpdate update1;
    update1.component = "C";
    update1.name = "L1";

    armarx::viz::data::LayerUpdate update2;
    update2.component = "C";
    update2.name = "L2";

    armarx::viz::data::LayerUpdates updates;
    updates.updates.push_back(update1);
    updates.updates.push_back(update2);

    buffer.addUpdates(std::move(updates));
    REQUIRE(buffer.getChangedLayers().size() == 2);
    REQUIRE(buffer.getChangedLayers().contains({"C", "L1"}));
    REQUIRE(buffer.getChangedLayers().contains({"C", "L2"}));

    auto update = buffer.getUpdate({"C", "L2"});
    REQUIRE(update.has_value());
    REQUIRE(update->name == "L2");

    update = buffer.getUpdate({"C", "L1"});
    REQUIRE(update.has_value());
    REQUIRE(update->name == "L1");

    update = buffer.getUpdate({"C", "L1"});
    REQUIRE(not update.has_value());
}

TEST_CASE("two components with a layer with the same name", "[RevisionBuffer]")
{
    avi::connection::RevisionBuffer buffer;

    armarx::viz::data::LayerUpdate update1;
    update1.component = "C1";
    update1.name = "L";

    armarx::viz::data::LayerUpdate update2;
    update2.component = "C2";
    update2.name = "L";

    armarx::viz::data::LayerUpdates updates;
    updates.updates.push_back(update1);
    updates.updates.push_back(update2);

    buffer.addUpdates(std::move(updates));
    REQUIRE(buffer.getChangedLayers().size() == 2);
    REQUIRE(buffer.getChangedLayers().contains({"C1", "L"}));
    REQUIRE(buffer.getChangedLayers().contains({"C2", "L"}));

    auto update = buffer.getUpdate({"C2", "L"});
    REQUIRE(update.has_value());
    REQUIRE(update->name == "L");

    update = buffer.getUpdate({"C1", "L"});
    REQUIRE(update.has_value());
    REQUIRE(update->name == "L");

    update = buffer.getUpdate({"C3", "L"});
    REQUIRE(not update.has_value());
}
